<?php

namespace Drupal\danse_content_moderation\Plugin\Danse;

use Drupal\Core\Entity\EntityMalformedException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\PayloadInterface;
use Drupal\danse\PluginBase;
use Drupal\danse_content_moderation\Payload;

/**
 * Plugin implementation of DANSE.
 *
 * @Danse(
 *   id = "content_moderation",
 *   label = @Translation("Content Moderation"),
 *   description = @Translation("Provides Content Moderation integration for DANSE.")
 * )
 */
class ContentModeration extends PluginBase {

  /**
   * {@inheritdoc}
   */
  public function assertPayload(PayloadInterface $payload): bool {
    return $payload instanceof Payload;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedSubscriptions($roles): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, FormStateInterface $form_state): array {
    return [];
  }


  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(EventInterface $event): Url {
    /** @var \Drupal\danse_content_moderation\Payload $payload */
    $payload = $event->getPayload();
    try {
      return $payload->getEntity()->toUrl();
    }
    catch (EntityMalformedException $e) {
      return Url::fromRoute('<front>');
    }
  }

  /**
   * Create Content Moderation DANSE event.
   *
   * @param \Drupal\danse_content_moderation\Payload $payload
   *   The payload.
   *
   * @return \Drupal\danse\Entity\EventInterface
   *   The DANSE event.
   */
  public function createContentModerationEvent(Payload $payload) {
    /** @var \Drupal\content_moderation\Entity\ContentModerationState $entity */
    $entity = $payload->getEntity();
    $topic = $entity->moderation_state->value;
    $parent_rev_id = $entity->content_entity_revision_id->value;
    $parent_entity_type = $entity->content_entity_type_id->value;
    /** @var \Drupal\node\Entity\Node $revision */
    $revision = \Drupal::entityTypeManager()->getStorage($parent_entity_type)->loadRevision($parent_rev_id);
    // Try to load the 'previous' revision.
    /** @var \Drupal\node\Entity\Node $last_version_node */
    $last_version_node = $this->getPreviousRevision($revision->id(), $parent_rev_id);
    $revision->last_revision = $last_version_node->getRevisionId();

    $label = $revision->label() . ' transitioned to ' . $topic;
    $revision_payload = new Payload($revision);
    return $this->createEvent($topic, $label, $revision_payload);
  }

  /**
   * @param $nid
   * @param $currently_revision_id
   *
   * @return false|mixed
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getPreviousRevision($nid, $currently_revision_id) {
    // Getting the node storage.
    /** @var \Drupal\node\NodeStorageInterface $node_storage */
    $node_storage = \Drupal::entityTypeManager()->getStorage('node');
    // Getting the node.
    $node = $node_storage->load($nid);

    // Get all revisions of the current node, in all languages.
    $revision_ids = $node_storage->revisionIds($node);

    $revisions_before = [];
    if (count($revision_ids) > 0) {
      // Loop through the list of revision ids, select the ones that have.
      // Same language as the current language AND are older than the current
      // deleted revision.
      foreach ($revision_ids as $vid) {
        // Compare revision using vid, the newer revision has bigger vid.
        if ($currently_revision_id - $vid > 0 ) {
          $revision = $node_storage->loadRevision($vid);
          array_push($revisions_before, $revision);
        }
      }

      // Sort revisions by comparing revisionId, newest revision first.
      usort($revisions_before, function ($rev1, $rev2) {
        if ($rev1->getRevisionId() == $rev2->getRevisionId()) {
          return 0;
        }
        return ($rev1->getRevisionId() < $rev2->getRevisionId()) ? -1 : 1;
      });
    }

    return reset($revisions_before);
  }

}
